<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Email\Models\EventEmail;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\Actions\AssociateToEvent;
use EventPortal\Domain\Util\Notification\SystemNotification;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Facades\Notification;
use Spatie\Snapshots\MatchesSnapshots;

class EmailSendTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test **/
    public function can_send_an_event_email_to_all_delegates()
    {
        Notification::fake();

        $event = factory(Event::class)->create([
            'name' => 'Hello World!',
        ]);
        factory(EventDelegate::class, 3)->create([
            'event_id' => $event->id,
        ]);
        $email = factory(EventEmail::class)
            ->state('invite')
            ->create([
                'event_id' => $event,
            ]);

        $this
            ->actingAsSuperAdmin()
            ->fromRoute('admin.dashboard.index')
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        Notification::assertSentTo(
            [$event->delegates->pluck('user')],
            SystemNotification::class,
            function (SystemNotification $notification) use ($event) {
                return $notification->subject === "You're invited to attend Hello World!"
                    && $notification->data->getEvent()->id === $event->id;
            }
        );
    }

    /** @test **/
    public function emails_can_be_sent_to_only_those_who_havent_already_been_sent_it()
    {
        $this->withoutExceptionHandling();

        Notification::fake();

        $event = factory(Event::class)->create([
            'name' => 'Hello World!',
        ]);
        $initialDelegate = factory(EventDelegate::class)->create([
            'event_id' => $event->id,
        ])->delegate->user;

        $email = factory(EventEmail::class)
            ->state('invite')
            ->create([
                'event_id' => $event,
            ]);

        // Send for the first time (just to initialDelegate)
        $this
            ->actingAsSuperAdmin()
            ->fromRoute('admin.event.index')
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        $secondDelegate = factory(EventDelegate::class)->create([
            'event_id' => $event->id,
        ])->delegate->user;

        // Send for the second time (just to secondDelegate, even though initialDelegate is still invited)
        $this
            ->actingAsSuperAdmin()
            ->fromRoute('admin.event.index')
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
                'only_remaining' => true,
            ]))
            ->assertOk();

        $this->assertCount(1, Notification::sent($initialDelegate, SystemNotification::class));
        $this->assertCount(1, Notification::sent($secondDelegate, SystemNotification::class));
    }

    /** @test **/
    public function delegates_are_marked_as_invited_when_the_invite_email_is_sent()
    {
        [$event, $otherEvent] = factory(Event::class, 2)->create();

        $eventDelegates = factory(EventDelegate::class, 3)
            ->state('candidate')
            ->create([
                'event_id' => $event->id,
            ]);

        $otherEventDelegates = factory(EventDelegate::class, 3)
            ->state('candidate')
            ->create([
                'event_id' => $otherEvent->id,
            ]);

        $email = factory(EventEmail::class)
            ->state('invite')
            ->create([
                'event_id' => $event->id,
            ]);

        $this
            ->actingAsSuperAdmin()
            ->fromRoute('admin.dashboard.index')
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        $eventDelegates->fresh()->pluck('status')->each(function ($delegate) {
            $this->assertEquals('invited', $delegate);
        });

        $otherEventDelegates->fresh()->pluck('status')->each(function ($otherDelegate) {
            $this->assertEquals('candidate', $otherDelegate);
        });
    }

    /** @test **/
    public function emails_are_sent_to_PA_if_PA_field_present()
    {
        Notification::fake();

        $event = factory(Event::class)
            ->state('published')
            ->create();
        Pathway::generate($event, Pathway::DEFAULT, 'Default pathway');

        $managedDelegate = factory(Delegate::class)
            ->state('snapshot')
            ->create([
                'pa_email' => 'pa@example.com',
            ]);

        (new AssociateToEvent())->execute($event, [
            $managedDelegate->id,
        ]);

        $email = factory(EventEmail::class)
            ->create([
                'event_id' => $event,
            ]);

        $this
            ->actingAsSuperAdmin()
            ->fromRoute('admin.dashboard.index')
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        Notification::assertSentTo(
            $managedDelegate->user,
            SystemNotification::class,
            function (SystemNotification $notification, $channels, $user) use ($managedDelegate) {
                return $user->routeNotificationForMail($notification) === [
                    'pa@example.com' => $managedDelegate->name,
                ];
            }
        );
    }

    /** @test **/
    public function emails_are_sent_to_delegate_if_PA_field_empty()
    {
        Notification::fake();

        $event = factory(Event::class)
            ->state('published')
            ->create();
        Pathway::generate($event, Pathway::DEFAULT, 'Default pathway');

        $standardDelegate = factory(Delegate::class)
            ->state('snapshot')
            ->create([
                'pa_email' => null,
            ]);

        (new AssociateToEvent())->execute($event, [
            $standardDelegate->id,
        ]);

        $email = factory(EventEmail::class)
            ->create([
                'event_id' => $event,
            ]);

        $this
            ->actingAsSuperAdmin()
            ->fromRoute('admin.dashboard.index')
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        Notification::assertSentTo(
            $standardDelegate->user,
            SystemNotification::class,
            function (SystemNotification $notification, $channels, $user) use ($standardDelegate) {
                return $user->routeNotificationForMail($notification) === [
                    $standardDelegate->user->email => $standardDelegate->name,
                ];
            }
        );
    }

    /** @test **/
    public function the_sender_and_reply_addresses_can_be_overridden()
    {
        Notification::fake();

        $event = factory(Event::class)->create();
        factory(EventDelegate::class, 3)->create([
            'event_id' => $event->id,
        ]);
        $email = factory(EventEmail::class)
            ->state('invite')
            ->create([
                'event_id' => $event,
                'sender_address' => 'override-sender@example.com',
                'reply_to_address' => 'override-reply-to@example.com',
            ]);

        $this
            ->actingAsSuperAdmin()
            ->fromRoute('admin.dashboard.index')
            ->followingRedirects()
            ->post(route('admin.email.send', [
                'event' => $event,
                'email' => $email,
            ]))
            ->assertOk();

        Notification::assertSentTo(
            [$event->delegates->pluck('user')],
            SystemNotification::class,
            function (SystemNotification $notification) use ($event) {
                return $notification->data->getEvent()->id === $event->id
                    && $notification->from === 'override-sender@example.com'
                    && $notification->replyTo === 'override-reply-to@example.com';
            }
        );
    }
}
